#!/usr/bin/perl
#
#(c) AngieSoft 2006 Lizenz GPL 
#mailto:angiesoft@gmx.net?subject=webcam Upload
#
#
#Script,erzeugt in Abhngigkeit des Sonnenstandes des angegebenen Ortes und einiger zu definierender Variable
#ein TRUE oder FALSE auf stdout
#zustzlich wird die index2.tmpl einmal tglich erzeugt
#der bash-script "cam-up" ruft zu diesem Zweck diesen Script auf (sh. Kommentare dort)
#
#die Formeln zur (einfachen) Berechnung des Sonnenauf- und -unterganges stammen von
#http://lexikon.astronomie.info/zeitgleichung/
#
#dieser script entstand, da meine Webcam von ConnecTec keine vernnftigen Nachtbilder liefert
#

use strict;
use Math::Trig;
use Time::localtime;

#---------Einstellungen-----------
my $nord=49.05; #geograf. Breite (in Dezimalgrad) und
my $east=11.183; #geogr. Lnge des Ortes der Kamera (in Dezimalgrad)
my $toleranz = 0.1; #+- 0.1 Std = 6 Min - cron muss den Script also mindestens alle 5' aufrufen (sh. bash-script "cam-up")
my $daemmerung = -6; #brgerliche Dmmerung - genaue Beschreibung sh. o.a. Webseite (Winkel der Sonne hinter Horizont)
#my $daemmerung = -50/60;  #geometrische Horizonthhe (~-50 Bogenminuten) - exakter Sonnenuntergang
my $camintervall = 1; #Uploadintervall in Dezimalstunden (also 0.25 heit alle 15') 
                      #wegen der Funktionsweise des Scripts ist <= 0.1 nicht sinnvoll (sollte aber trotzdem funzen)
my $zone=1; #Zeitzone in Std. vor UTC (Winterzeit) Sommerzeitumstellung macht der Script


############SCRIPT#############
#globale Variable bzw. Konstanten
my $RAD = pi/180.0;
my $Daemmerung = $daemmerung * $RAD; #Dmmerung in Rad
my $Breite = $nord * $RAD; #geogr. Breite in Radiant
my $tm = localtime;
if ($tm->isdst) { $zone += 1; } # +1, wenn Sommerzeit

###Funktionen####
sub zeitdifferenz
#12 Uhr minus Sonnenaufgang / Untergang (Dmmerung) 
{
     return 12.0 * acos((sin($Daemmerung) - sin($Breite) * sin($_[0])) / (cos($Breite) * cos($_[0])))/pi; 	

}

sub zeitgleichung
#Zeitversatz zwischen wahrer Ortszeit und mittlerer Ortszeit
{ 
	return -0.1752 * sin(0.033430 * $_[0] + 0.5474) - 0.1340 * sin(0.018234 * $_[0] - 0.1939); 
	
}

sub frac
#Nachkommaanteil einer reelen Zahl
{
  return $_[0] - int($_[0])
}

sub zeitstring
#macht aus einem dezimalen Stundenbetrag eine Zeit entspr. HH:MM
{
   return sprintf("%02d:%02d", int($_[0]),int((frac($_[0]) * 60) + 0.5));
}

####main####
my $deklination = 0.40954 * sin(0.0172 * ($tm->yday - 79.35)); #Zenithhe der Sonne in Rad

#gerundete Berechnung Sonnenauf- und untergang fr Webcam aktiv
my $Startwebcam = 12 - zeitdifferenz($deklination) - zeitgleichung($tm->yday)  - $east / 15.0 + $zone;
my $Endwebcam   = 12 + zeitdifferenz($deklination) - zeitgleichung($tm->yday)  - $east / 15.0 + $zone;
my $Startstr  = zeitstring($Startwebcam); #fr die Ausgabe im Template
my $Endstr = zeitstring($Endwebcam);

my $aktmin  = $tm->min / 60 + $tm->sec / 3600;
my $intervall = frac($aktmin / $camintervall); #Minuten auf Intervall hochrechnen
my $aktzeit = $tm->hour + $aktmin;

my $webcamaktiv = (( $aktzeit => $Startwebcam ) & ( $aktzeit <= $Endwebcam )); #nur innerhalb dieses Zeitrahmens

#print "Start: $Startstr Stop: $Endstr Dmmerung: $Daemmerung\n";
if ( $webcamaktiv ) {
  my $erstefrueh = $aktzeit - $Startwebcam ; #ist erreicht, wenn $erstefrueh > 0
  my $letzteabends =  $Endwebcam - $aktzeit ; #ist erreicht, wenn $letzteabends < 0
  my $webcamupdate = (( ($intervall > 0.02) & ($intervall < $toleranz )) | ( $erstefrueh < $toleranz) | ( $letzteabends < $toleranz));
    if ( $webcamupdate ) {
     if ( $erstefrueh < $toleranz) { 
       #index2.tmpl nur neu schreiben, wenn neuer Tag und der ist mit obigen Bedingungen erreicht
       open(INDEX2,">index2.tmpl") || die "index2.tmpl kann nicht geschrieben werden" ;
#<!-- template Teil 2 hier am Beispiel meiner Webcam http://www.angiesoft.de/greding/ -->
       print INDEX2 qq(<p><font size='-2'>st&uuml;ndliche Aktualisierung von $Startstr - $Endstr Uhr</font>\n);
       print INDEX2 qq(<br>eingetragen bei <a href='http://www.earthcam.com'>EarthCam</a><p>);
       print INDEX2 qq(<a href="http://angiesoft.homelinux.net:8080/" target="_new">Livebild</a>);
       print INDEX2 qq(<fontsize="-2"> Login mit User = gast - Passwort = -ohne-</font>);
       print INDEX2 qq(<br><font size='2'>Tipp: Klick auf's Bild ....</center>);
       print INDEX2 qq(</td><td><iframe width="160" height="170" marginheight="0" marginwidth="0" frameborder="0" src="http://www.wetteronline.de/cgi-bin/homecif?WMO=10777&PLZN=Greding&L=de&BGC=00C0FF&B=0"></iframe>);
       print INDEX2 qq(</td></tr></table></body><html>\n);
       close(INDEX2);
     }
     printf stdout "TRUE" 
     } else  {  printf stdout "FALSE" }
  } else {
       printf stdout "FALSE" 
  }
  
